/*************************************************
* Zeus CMD - OpenGL ES Tutorial 25 : Reflections *
* By Grant James (ZEUS)                          *
* http://www.zeuscmd.com                         *
*************************************************/
#pragma comment(lib, "libGLES_CM.lib")

#define GLUTES_STATIC

#include <GLES/glutes.h>
#include <GLES/glues.h>
#include <GLES/glues.c>

float xrot = 0.0f;
float yrot = 0.0f;
float yval = 1.0f;
bool yup = true;

GLfloat floorVertices[] = {
	-3.0f, 0.0f, 3.0f,
	 3.0f, 0.0f, 3.0f,
	-3.0f, 0.0f,-3.0f,
	 3.0f, 0.0f,-3.0f
};

GLfloat box[] = {
	// FRONT
	-0.5f, -0.5f,  0.5f,
	 0.5f, -0.5f,  0.5f,
	-0.5f,  0.5f,  0.5f,
	 0.5f,  0.5f,  0.5f,
	// BACK
	-0.5f, -0.5f, -0.5f,
	-0.5f,  0.5f, -0.5f,
	 0.5f, -0.5f, -0.5f,
	 0.5f,  0.5f, -0.5f,
	// LEFT
	-0.5f, -0.5f,  0.5f,
	-0.5f,  0.5f,  0.5f,
	-0.5f, -0.5f, -0.5f,
	-0.5f,  0.5f, -0.5f,
	// RIGHT
	 0.5f, -0.5f, -0.5f,
	 0.5f,  0.5f, -0.5f,
	 0.5f, -0.5f,  0.5f,
	 0.5f,  0.5f,  0.5f,
	// TOP
	-0.5f,  0.5f,  0.5f,
	 0.5f,  0.5f,  0.5f,
	 -0.5f,  0.5f, -0.5f,
	 0.5f,  0.5f, -0.5f,
	// BOTTOM
	-0.5f, -0.5f,  0.5f,
	-0.5f, -0.5f, -0.5f,
	 0.5f, -0.5f,  0.5f,
	 0.5f, -0.5f, -0.5f,
};

GLfloat boxColors[] = {
	// FRONT
	 0.0f, 0.0f, 1.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	 0.0f, 0.0f, 1.0f, 0.0f,
	// BACK
	 1.0f, 0.0f, 0.0f, 0.0f,
	 0.0f, 0.0f, 1.0f, 0.0f,
	 0.0f, 0.0f, 1.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	// LEFT
	 0.0f, 0.0f, 1.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	 0.0f, 0.0f, 1.0f, 0.0f,
	// RIGHT
	 0.0f, 0.0f, 1.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	 0.0f, 0.0f, 1.0f, 0.0f,
	// TOP
	 1.0f, 0.0f, 0.0f, 0.0f,
	 0.0f, 0.0f, 1.0f, 0.0f,
	 0.0f, 0.0f, 1.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	// BOTTOM
	 0.0f, 0.0f, 1.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	 1.0f, 0.0f, 0.0f, 0.0f,
	 0.0f, 0.0f, 1.0f, 0.0f
};

void init()
{
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepthf(1.0f);
	glClearStencil(0);

	glEnableClientState(GL_VERTEX_ARRAY);
	glColorPointer(4, GL_FLOAT, 0, boxColors);

	glEnable(GL_CULL_FACE);
	glShadeModel(GL_SMOOTH);
}

void drawCube()
{
	glPushMatrix();

	glEnableClientState(GL_COLOR_ARRAY);

	glTranslatef(0.0f, yval, 0.0f);

	glRotatef(xrot, 1.0f, 0.0f, 0.0f);
	glRotatef(yrot, 0.0f, 1.0f, 0.0f);

	glVertexPointer(3, GL_FLOAT, 0, box);

	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glDrawArrays(GL_TRIANGLE_STRIP, 4, 4);

	glDrawArrays(GL_TRIANGLE_STRIP, 8, 4);
	glDrawArrays(GL_TRIANGLE_STRIP, 12, 4);

	glDrawArrays(GL_TRIANGLE_STRIP, 16, 4);
	glDrawArrays(GL_TRIANGLE_STRIP, 20, 4);

	glDisableClientState(GL_COLOR_ARRAY);

	glPopMatrix();
}

void display()
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT
		| GL_STENCIL_BUFFER_BIT);
	glLoadIdentity();

	gluLookAtf(
		0.0f, 3.0f, 7.0f,
		0.0f, 0.0f, 0.0f,
		0.0f, 1.0f, 0.0f);

	glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
	glDepthMask(GL_FALSE);
	glEnable(GL_STENCIL_TEST);
	glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
	glStencilFunc(GL_ALWAYS, 1, 1);

	// Floor
	glVertexPointer(3, GL_FLOAT, 0, floorVertices);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glDepthMask(GL_TRUE);

	glStencilFunc(GL_EQUAL, 1, 1);
	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);

	// Cube
	glPushMatrix();
	{
		glScalef(1.0f, -1.0f, 1.0f);

		glCullFace(GL_FRONT);

		drawCube();

		glCullFace(GL_BACK);
	}
	glPopMatrix();

	glDisable(GL_STENCIL_TEST);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// Floor
	glVertexPointer(3, GL_FLOAT, 0, floorVertices);
	glColor4f(1.0f, 1.0f, 1.0f, 0.4f);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	glDisable(GL_BLEND);

	drawCube();

	glFlush ();
	glutSwapBuffers();
}

void idle()
{
	xrot += 2.0f;
	yrot += 3.0f;

	yval += yup ? 0.02f : -0.02f;

	if (yval > 2.0f)
		yup = false;
	if (yval < 1.0f)
		yup = true;

	glutPostRedisplay();
}

void keyboard2(int key, int x, int y)
{
	switch(key)
	{
	case GLUT_KEY_UP : exit(0); break;
	}
}

void reshape(int width, int height)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glViewport(0, 0, width, height);
	gluPerspectivef(45.0f, 1.0f * width / height, 1.0f, 100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void menu(int entry)
{
	switch(entry)
	{
	case 1 : exit(0); break;
	}
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayString("stencil>=1 rgb depth double");
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_STENCIL);

	glutCreateWindow("25 - Reflections");

	init();

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutSpecialFunc(keyboard2);

	glutIdleFunc(idle);

	glutCreateMenu(menu);
	glutAddMenuEntry("Quit", 1);

	glutAttachMenu(GLUT_LEFT_BUTTON);
	
	glutMainLoop();

	return 0;
}
